use crate::{domain, service};
use serde::{Deserialize, Serialize};
use std::net;

#[derive(Serialize, Deserialize, Clone, PartialEq)]
#[serde(tag = "kind")]
pub enum ConfigDNSRecord {
    A { addr: net::Ipv4Addr },
    AAAA { addr: net::Ipv6Addr },
    CNAME { name: domain::Name },
}

impl From<ConfigDNSRecord> for domain::checker::DNSRecord {
    fn from(r: ConfigDNSRecord) -> Self {
        match r {
            ConfigDNSRecord::A { addr } => Self::A(addr),
            ConfigDNSRecord::AAAA { addr } => Self::AAAA(addr),
            ConfigDNSRecord::CNAME { name } => Self::CNAME(name),
        }
    }
}

#[derive(Deserialize, Serialize, Clone)]
pub struct Config {
    pub passphrase: String,

    #[serde(default)]
    pub dns_records: Vec<ConfigDNSRecord>,

    #[serde(default)]
    pub http: service::http::Config,

    #[serde(default)]
    pub gemini: service::gemini::Config,
}
